% Reduced Rank Extrapolation
% X is a matrix whos columns are the current solution vectors
%
% By Guy Rosman, 2008
%
% for more details, see:
% "G. Rosman, A. M. Bronstein, M. M. Bronstein, R. Kimmel, Manifold
% Analysis by Topologically Constrained Isometric Embedding, World
% Enformatika Conference on Machine Learning and Pattern Recognition (MLPR), 2006
%
% ANY ACADEMIC USE OF THIS CODE MUST CITE THE ABOVE REFERENCES. 
% ANY COMMERCIAL USE PROHIBITED WITHOUT WRITTEN CONSENT OF THE AUTHORS. 
% PLEASE CONTACT THE AUTHORS FOR LICENSING TERMS. 
% PROTECTED BY INTERNATIONAL INTELLECTUAL PROPERTY LAWS AND PATENTS PENDING.
%
function [s,accelStats]=RRE(X)
accelStats=[];
U=X(:,2:end)-X(:,1:(end-1));
[Q,R]=mgs(U);
clear U;
R=triu(R);
k=size(R,2);
e=ones([k,1]);
d=R\(R'\e);
if (nargout>1)
    accelStats.LS_residual=norm(R'*R*d-e);
    accelStats.Ucn=cond(U);
    svdU=svd(U);
    accelStats.Ugap=svdU(1)/svdU(2);
    accelStats.Unorm=norm(U);
    accelStats.Upnorm=norm(pinv(U));

end
gam=d/sum(d);
xi(1)=1-gam(1);
for i = 1:(length(gam)-1)
    xi(i+1)=xi(i)-gam(i+1);
end
s=X(:,1)+Q*(R*xi');
end
function [Q, R] = mgs(V)
n=size(V,2);
R = zeros(n,n);
for i=1:n
    R(i,i) = norm(V(:,i));
    V(:,i) = V(:,i)/R(i,i);
    if (i < n)
        for j = i+1:n
            R(i,j) = V(:,i)' * V(:,j);
            V(:,j) = V(:,j) - R(i,j) * V(:,i);
        end
    end
end
Q = V;
end