function [ys, stats] = computeTCIE(X,K, d,options)
% Topologically Constrained Isometric Embedding
% By Guy Rosman, 2008
%
% for more details, see:
% "G. Rosman, A. M. Bronstein, M. M. Bronstein, R. Kimmel, Manifold
% Analysis by Topologically Constrained Isometric Embedding, World
% Enformatika Conference on Machine Learning and Pattern Recognition (MLPR), 2006
%
% ANY ACADEMIC USE OF THIS CODE MUST CITE THE ABOVE REFERENCES. 
% ANY COMMERCIAL USE PROHIBITED WITHOUT WRITTEN CONSENT OF THE AUTHORS. 
% PLEASE CONTACT THE AUTHORS FOR LICENSING TERMS. 
% PROTECTED BY INTERNATIONAL INTELLECTUAL PROPERTY LAWS AND PATENTS PENDING.
%
% Usage: 
% [ys, stats] = computeTCIE(X,K, d,options)
% inputs:
% X - represents either the original coordinates, or (if
% options.distanceMatrix==1), the distance matrix between the various data
% points.
% K - denotes how many neighbors to take in the proximity graph. 12 is
% customary.
% d - denotes the embedding space dimension
% options.weights determines the type of weights used:
% 1 means geodesics which do not pass through boundary points
% 0 means usual ISOMAP weights (all ones)
% options.ITERS denotes the maximal number of iterations per scale;
% options.minDStress signifies the minimum relative change in stress before convergence stops
% options.min_halt_iter denotes the minimal number of iterations to run
% outputs:
% ys - the mapped images of the points in X
% stats - statistics on the algorithm
N=size(X,1);
if (isfield(options,'weights')==0)
    options.weights=1;
end
if ~isfield(options,'display')
    options.display = 1;
end
if ~isfield(options,'verbose')
    options.verbose = 1;
end
if (isfield(options,'flatten_in_advance')==0)
    flatten_in_advance=0;
else
    flatten_in_advance=options.flatten_in_advance;
end
if (isfield('options','distanceMatrix') && (options.distanceMatrix==1))
    D=X;clear X;
else
    if (size(X,2)<d)
        error('The original, embedding space, dimension of X most be <= d');
    end
D=L2_distance(X',X');
end
K2=K;
if isfield(options,'K2')
    K2=options.K2;
end

D_full=D;
D_neighbors=zeros(size(D));
clear D;
for i=1:N
    %     d = feval(d_func,i);
    dd = D_full(i,:);
    %     if n_fcn == 'k'
    [c,b] = sort(dd);
    D_neighbors(i,b(1:(K+1))) = c(1:(K+1));
end
D=max(D_neighbors,D_neighbors');
D = max(D,D');    %% Make sure distance matrix is symmetric
clear D_neighbors;
if (find(sum(~isinf(D))<2)>0)
    error('Isolated points found');

end

if (find(sum(~isinf(D))<2)>0)
    error('Isolated points found');
end

if (flatten_in_advance)
[D2,pD] = dijkstraWBoundary(full(D),zeros([1 N]));
id=detectBoundaryPointsN5(D2,d,K2,1);
clear D2 pD;
else
    id=detectBoundaryPointsN5(D_full,d,K2,0);
end
stats.id=id;

wx=ones([1 N]);wx(id)=0;
[D,pD] = dijkstraWBoundary(full(D),double(wx));
pD=double(~pD);
D=min(D,D');D((D<1E-10))=0;
clear id;
if options.weights==1
    Ws=zeros(size(D));
    KK=12;

    for i = 1: length(D)
        [sDi,id]=sort(D(:,i));
        idsk=id(2:KK);
        Ws(idsk,i)=1;
        Ws(i,idsk)=1;
    end
    Ws=min(2,max((1.00000001-pD),Ws));

    clear sD id 
else
    Ws=ones(size(D));
end
pidx=find(pD(:)==0);
pidx2=find((D(:)~=0)&(pD(:)==0));
clear pD

minWeight=0;
id=find(wx==0);
[ys,yss,stats]=weightedSMACOFMS(D,d,(Ws+minWeight)/(sum(sum((Ws+minWeight)))),id,options,1e-7,X');
stats.id=id;
stats.wx=wx;
stats.pidx=pidx;
stats.pidx2=pidx2;
stats.stress=stats.stress(end);
stats.flatten_in_advance=flatten_in_advance;
stats.yss2=yss;
if (exist('idBoundary','var'))
    stats.idBoundary=idBoundary;
end

return;







