// computeDijkstraStress.cpp
// Will hold a computation of the stress between a matrix D and the matrix resulting 
// from a dijkstra computation, as well as its derivatives w.r.t the coordinates of 
// the points creating the distances for the Dijkstra algorithm
// (under construction)	
#include <map>
using std::map;

#include <vector>
using std::vector;
using std::pair;
#include "dijkFibHeap.h"
#include "mex.h"


FibHeap *theHeap;

template<class T> T& accessArray(T* a,int M, int N,int i, int j){
    return a[i+M*j];
}

template<class T> T& accessArray(const T*a,int M, int N, int i, int j) {
    return a[i+M*j];
}

void mexFunction(
int          nlhs,
mxArray      *plhs[],
int          nrhs,
const mxArray *prhs[]
)
{
    if ((nrhs<2)||(nlhs>2)) {
        mexPrintf("syntax: [dijkstra_D,W]=dijkstraWBoundary(D,w)\n");
        mexPrintf("        D(i,j)>0 for non-neighboring points and if i==j\n");
        mexPrintf("        w is a vector with w(i)==1 if w is a boundary point\n");
        return;
    }
    double   INF,SMALL;
    int num_of_vertices;
    num_of_vertices=mxGetM(prhs[0]);
    double *D=mxGetPr(prhs[0]);
    double *boundary=mxGetPr(prhs[1]);
	plhs[0]=mxCreateDoubleMatrix(num_of_vertices,num_of_vertices,mxREAL);
	plhs[1]=mxCreateDoubleMatrix(num_of_vertices,num_of_vertices,mxREAL);
	double *dijkstra_D=mxGetPr(plhs[0]);
	double *W=mxGetPr(plhs[1]);

    map<int,map<int, double> > edges;
    INF   = mxGetInf();
    SMALL = mxGetEps();
    theHeap=new FibHeap;
    for(int i(0);i<num_of_vertices;++i){
        accessArray(dijkstra_D,num_of_vertices,num_of_vertices,i,i)=0;
                accessArray(W,num_of_vertices,num_of_vertices,i,i)=1;
        for(int j(i+1);j<num_of_vertices;++j){
                accessArray(W,num_of_vertices,num_of_vertices,i,j)=1;
                accessArray(W,num_of_vertices,num_of_vertices,j,i)=1;

            double dist_ij=accessArray(D,num_of_vertices,num_of_vertices,i,j);
            if (dist_ij>0){
                (edges[i])[j]=dist_ij;
                (edges[j])[i]=dist_ij;
                accessArray(dijkstra_D,num_of_vertices,num_of_vertices,i,j)=dist_ij;
                accessArray(dijkstra_D,num_of_vertices,num_of_vertices,j,i)=dist_ij;
            }else{
                accessArray(dijkstra_D,num_of_vertices,num_of_vertices,i,j)=INF;
                accessArray(dijkstra_D,num_of_vertices,num_of_vertices,j,i)=INF;
            }
            
        }
    }    
    for(int src=0;src<num_of_vertices;++src){
        accessArray(dijkstra_D,num_of_vertices,num_of_vertices,src,src)=0;
        for(int j(src);j<num_of_vertices;++j){
            double dist_ij=accessArray(D,num_of_vertices,num_of_vertices,src,j);
            if ((dist_ij>0)||(j==src)){
                accessArray(dijkstra_D,num_of_vertices,num_of_vertices,src,j)=dist_ij;
                accessArray(dijkstra_D,num_of_vertices,num_of_vertices,j,src)=dist_ij;
            }else{
                accessArray(dijkstra_D,num_of_vertices,num_of_vertices,src,j)=INF;
                accessArray(dijkstra_D,num_of_vertices,num_of_vertices,j,src)=INF;
            }
            
        }
	
    HeapNode *Min;
    HeapNode Temp;
    vector<HeapNode*> heapnodes(num_of_vertices,0);
        //mexPrintf("Src = %d\n",src);
        vector<double> ds(num_of_vertices,INF);
        ds[src]=0;
        for (int i=0;i<num_of_vertices;++i){
            HeapNode *newNode;
            if (i==src){
                newNode=new HeapNode(0,i);
            } else {
                if (edges[src].find(i)==edges[src].end()){
                    newNode=new HeapNode(INF,i);
                } else {
                    newNode=new HeapNode((edges[src])[i],i);
                    
                }
            }
            theHeap->Insert(newNode);
            heapnodes[i]=newNode;
        }
        // Insert 0 then extract it.  This will cause the
        // Fibonacci heap to get balanced.
        theHeap->Insert(&Temp);
        theHeap->ExtractMin();
        
        while (theHeap->GetNumNodes()>0){
            HeapNode *currentMinimum = (HeapNode *)theHeap->ExtractMin();
            int idx=currentMinimum->GetIndexValue();
            map<int, double> &currentEdges=edges[idx];
            for (map<int,double>::iterator iter=currentEdges.begin();iter!=currentEdges.end();++iter){
                int neigh_idx=iter->first;
                double dist_si=accessArray(dijkstra_D,num_of_vertices,num_of_vertices,src,idx);
                double dist_ij=iter->second;
                double dist_sj=accessArray(dijkstra_D,num_of_vertices,num_of_vertices,src,neigh_idx);
                if (dist_sj>(dist_si+dist_ij)){
                    double new_dist=dist_si+dist_ij;
                    if (((accessArray(boundary,num_of_vertices,1,idx,0)>0) &&(idx!=src))||(accessArray(W,num_of_vertices,num_of_vertices,src,idx)==0)){
                        accessArray(W,num_of_vertices,num_of_vertices,src,neigh_idx)=0;
                        //accessArray(W,num_of_vertices,num_of_vertices,neigh_idx,src)=0;
					} else {
                        accessArray(W,num_of_vertices,num_of_vertices,src,neigh_idx)=1;
					}
                    accessArray(dijkstra_D,num_of_vertices,num_of_vertices,src,neigh_idx)=new_dist;
                   // accessArray(dijkstra_D,num_of_vertices,num_of_vertices,neigh_idx,src)=new_dist;
                    HeapNode *updatedNode=heapnodes[neigh_idx];
                    HeapNode newNode(new_dist,neigh_idx);
                    //mexPrintf("Updating node %d using node %d\n",neigh_idx,idx);
                    theHeap->DecreaseKey(updatedNode,newNode);
                }
            }
        }
// 		for(int i=0;i<num_of_vertices;++i){
// 			accessArray(result_D,num_of_vertices,num_of_vertices,src,i)=accessArray(dijkstra_D,num_of_vertices,num_of_vertices,src,i);
// 		}
    }
    delete theHeap;
}