% Perform farthest point sampling
% inputs:
% D - A distance matrix
% W - A weights matrix
% N - The number of points to sample
% boundaryPts - Detected boundary points
% idx - The initially selected points. Default - point 1
% By Guy Rosman, 2008
%
% for more details, see:
% "G. Rosman, A. M. Bronstein, M. M. Bronstein, R. Kimmel, Manifold
% Analysis by Topologically Constrained Isometric Embedding, World
% Enformatika Conference on Machine Learning and Pattern Recognition (MLPR), 2006
%
% ANY ACADEMIC USE OF THIS CODE MUST CITE THE ABOVE REFERENCES. 
% ANY COMMERCIAL USE PROHIBITED WITHOUT WRITTEN CONSENT OF THE AUTHORS. 
% PLEASE CONTACT THE AUTHORS FOR LICENSING TERMS. 
% PROTECTED BY INTERNATIONAL INTELLECTUAL PROPERTY LAWS AND PATENTS PENDING.
%
function [idx,epsilon] = fps_weighted(D,W,N,boundaryPts,idx);
INF=1e5;
minWeight=1E-5;
if (exist('idx')~=1)
%     idx = [1:500]';
idx=1;
else
    idx=idx(:);% to ensure shape
end
iidx=setdiff([1:length(D)],idx);
d   = min((W(idx,iidx)<minWeight)*INF+D(idx,iidx),[],1);   % d should always be the distance from each point to the set idx (closest point)

epsilon = [max(D(idx,iidx).*(D(idx,iidx)<INF))]';    % if idx contains a single seed element, we have epsilon as the sampling radius. Otherwise the first values of epsilon contain their maximal distance to other points
nonBoundaryPts=setdiff([1:length(D)],boundaryPts);
for k = 1:N-1,
   nidx=find(ismember(iidx,nonBoundaryPts));
   if (length(nidx)==0)
       nidx=1:length(iidx);
   end
    [m,i] = max(d(nidx));i=iidx(nidx(i));
%     d1 = min([d; (W(i,iidx)<minWeight)*INF+D(i,iidx)],[],1);
    d1 = min(d, [(W(i,iidx)<minWeight)*INF+D(i,iidx)]);

    d=d1;
    try
    d(find(iidx==i))=[];
    idx = [idx; i];iidx(find(iidx==i))=[];
    catch
        i
    end
    epsilon = [epsilon; m];
end

%epsilon = [epsilon; max(d)];
