% Generate the swiss roll, sampled using farthest point sampling
% (for internal use only)
% By Guy Rosman, 2008
%
% for more details, see:
% "G. Rosman, A. M. Bronstein, M. M. Bronstein, R. Kimmel, Manifold
% Analysis by Topologically Constrained Isometric Embedding, World
% Enformatika Conference on Machine Learning and Pattern Recognition (MLPR), 2006
%
% ANY ACADEMIC USE OF THIS CODE MUST CITE THE ABOVE REFERENCES. 
% ANY COMMERCIAL USE PROHIBITED WITHOUT WRITTEN CONSENT OF THE AUTHORS. 
% PLEASE CONTACT THE AUTHORS FOR LICENSING TERMS. 
% PROTECTED BY INTERNATIONAL INTELLECTUAL PROPERTY LAWS AND PATENTS PENDING.
%
function [X,TH,idealD]=generateRollNoise(N,sigma,BOUNDS,ratio,flatten);
if (exist('BOUNDS')==0)
    mt=5;Mt=6.5;mh=6;Mh=12;st=5.5;St=6;sh=8;Sh=10;
    BOUNDS=[mt Mt mh Mh st St sh Sh];
    ratio=1;
end
if (exist('flatten')~=1)
    flatten=0;
end
mt=BOUNDS(1);Mt=BOUNDS(2);mh=BOUNDS(3);Mh=BOUNDS(4);
st=BOUNDS(5);St=BOUNDS(6);sh=BOUNDS(7);Sh=BOUNDS(8);
X=[];
ts=[];
hs=[];
K=1;
for i = 1 : N
    found=0;
    if (length(ts)>=3)
        TH=[ts(:),hs(:)];
        [V,C]=voronoin(TH);
        rvC=zeros(1,size(V,1));
        % get a listing, for each vertex, of a cell it belongs to
        for j = 1:size(C,1)
            rvC(C{j}(:))=j;
        end
        dVs=[];
        for j = 1:size(V,1)
            dV=norm([TH(rvC(j),:)-V(j,:)]);
            if (isLegit(V(j,:),BOUNDS))
                dVs=[dVs;dV];
            else
                dVs=[dVs;inf];
            end
        end
        dVs(find(isinf(dVs)))=-inf;
        [dV,minV]=max(dVs);
        tnew=V(minV,1);hnew=V(minV,2);
        [tnew,hnew]=bound(tnew,hnew,BOUNDS);
    else% if (length(ts)<=2)
        while 1
            tnew = (3*pi/2)*(mt+(Mt-mt)*rand([1,K]));  hnew = mh+(Mh-mh)*rand([1,K]);
            if(isLegit([tnew hnew],BOUNDS))
                break;
            end
        end
    end
    tooClose=0;
    n=randn([1 3])*sigma;
    %     x=(tnew-4*(3*pi/2))/(3*pi/2)*cos(tnew/2)+n(1);
    %     z=(tnew-4*(3*pi/2))/(3*pi/2)*sin(tnew/2)+n(3);
    if (~flatten)
        x=cos(tnew/2)+n(1);
        z=sin(tnew/2)+n(3);
        y=hnew/2*ratio+n(2);
    elseif(flatten==1)
        x=tnew/2+n(1);
        z=n(3);
        y=hnew/2*ratio+n(2);
    elseif(flatten==2)
        x=tnew/2+n(1);
        z=10*exp(-((hnew-8)^2+((tnew-27))^2)*1.2)+n(3);
        y=hnew/2*ratio+n(2);
    elseif(flatten==3)
        x=(1+tnew)/12*cos(tnew/2)+n(1);
        z=(1+tnew)/12*sin(tnew/2)+n(3);
        y=hnew/2*ratio+n(2);
    elseif(flatten==4)
        x=(1+tnew-3*pi/2*mt)/12*cos(tnew/2)+n(1);
        z=(1+tnew-3*pi/2*mt)/12*sin(tnew/2)+n(3);
        y=hnew/8*ratio+n(2);
    elseif(flatten==5)
        x=(1+tnew-3*pi/2*mt)/12*cos(tnew)+n(1);
        z=(1+tnew-3*pi/2*mt)/12*sin(tnew)+n(3);
        y=hnew/8*ratio+n(2);
    elseif(flatten==6)
        x=(1+tnew-3*pi/2*mt)/12*cos(tnew*1.5)+n(1);
        z=(1+tnew-3*pi/2*mt)/12*sin(tnew*1.5)+n(3);
        y=hnew/8*ratio+n(2);
    elseif(flatten==7)
        x=(tnew-3.5*pi/2*mt+hnew*0.7)/12*cos(tnew*1)+n(1);
        z=(tnew-3.5*pi/2*mt+hnew*0.7)/12*sin(tnew*1)+n(3);
        y=n(2);
    end
    ts=[ts, tnew];
    hs=[hs, hnew];
    X=[X,[x;y;z]];

    if (mod(i,5)==0)
        plot(ts,hs,'.'); title(sprintf('i=%d',i));axis image;drawnow;
    end

end
g=[1/4 0;0 ratio^2/4];
TH=[ts(:),hs(:)];
idealD=L2_g_distance(TH',TH',0,g);
end
function res=isLegit(dV,BOUNDS)
mt=BOUNDS(1);Mt=BOUNDS(2);mh=BOUNDS(3);Mh=BOUNDS(4);
st=BOUNDS(5);St=BOUNDS(6);sh=BOUNDS(7);Sh=BOUNDS(8);
tnew=dV(1);
hnew=dV(2);
res=~(((tnew > st*3*pi/2)&(tnew < St*3*pi/2)) &(tnew>=mt*3*pi/2)&(tnew<=Mt*3*pi/2) & ((hnew > sh) & (hnew <Sh))&(hnew>=mh) &(hnew<=Mh));
end
function [t,h]=bound(t,h,BOUNDS)
mt=BOUNDS(1);Mt=BOUNDS(2);mh=BOUNDS(3);Mh=BOUNDS(4);
t=max((3*pi/2)*mt,min(t,(3*pi/2)*(Mt)));
h=max(mh,min(Mh,h));
end