% Invert a matrix using Conjugate Gradients
% By Guy Rosman, 2008
%
% for more details, see:
% "G. Rosman, A. M. Bronstein, M. M. Bronstein, R. Kimmel, Manifold
% Analysis by Topologically Constrained Isometric Embedding, World
% Enformatika Conference on Machine Learning and Pattern Recognition (MLPR), 2006
%
% ANY ACADEMIC USE OF THIS CODE MUST CITE THE ABOVE REFERENCES. 
% ANY COMMERCIAL USE PROHIBITED WITHOUT WRITTEN CONSENT OF THE AUTHORS. 
% PLEASE CONTACT THE AUTHORS FOR LICENSING TERMS. 
% PROTECTED BY INTERNATIONAL INTELLECTUAL PROPERTY LAWS AND PATENTS PENDING.
%
function [x_j,stats]=linearPCG(applyA,getB,x0,applyM,displayFunction,MAX_ITER,stats0,MIN_RES)
rn=[];
rn2=[];
if (exist('stats0','var'))
    stats=stats0;
else
    stats=[];
end
if (exist('MIN_RES','var')==0)
    MIN_RES=0;
end
x_j=x0;
r_j=getB(x_j)-applyA(x_j);
rn(end+1)=norm(r_j(:));
z_j=applyM(r_j);
p_j=z_j;
for j=1:MAX_ITER
    if (mod(j,20)==0)
% z_j=applyM(r_j);
    end
    q_j=applyA(p_j);
    tmp_denom=q_j(:)'*p_j(:);
    if abs(tmp_denom)<1e-10
        tmp_denom=1e-10;
    end
    alpha_j=(z_j(:)'*r_j(:))/(tmp_denom);
    x_jp1=x_j+alpha_j*p_j;
    r_jp1=r_j-alpha_j*applyA(p_j);
    r_jp2=x_jp1-x_j;
%    r_jp1=getB(x_j)-applyA(x_jp1);
    z_jp1=applyM(r_jp1);
    tmp_denom=r_j(:)'*z_j(:);
    if (tmp_denom==0)
        tmp_denom=1e-10;
    end
    beta_jp1=r_jp1(:)'*z_jp1(:)/(tmp_denom);
    
    p_jp1=z_jp1+beta_jp1*p_j;
    p_j=p_jp1;
    r_j=r_jp1;
    z_j=z_jp1;
    x_j=x_jp1;
rn(end+1)=norm(r_j(:));
rn2(end+1)=norm(r_jp2(:));
% rn(end)
if (rn(end)<max(MIN_RES,1e-15))
    break;
end
    stats.rn=rn;
    stats.rn2=rn2;
    displayFunction(x_j,stats,j);
end
rn;
end
    