% Example code for running Topologically Constrained Isometric Embedding
% Creates a Swiss roll with a hole in it, and maps it using TCIE
% By Guy Rosman, 2008
%
% for more details, see:
% "G. Rosman, A. M. Bronstein, M. M. Bronstein, R. Kimmel, Manifold
% Analysis by Topologically Constrained Isometric Embedding, World
% Enformatika Conference on Machine Learning and Pattern Recognition (MLPR), 2006
%
% ANY ACADEMIC USE OF THIS CODE MUST CITE THE ABOVE REFERENCES. 
% ANY COMMERCIAL USE PROHIBITED WITHOUT WRITTEN CONSENT OF THE AUTHORS. 
% PLEASE CONTACT THE AUTHORS FOR LICENSING TERMS. 
% PROTECTED BY INTERNATIONAL INTELLECTUAL PROPERTY LAWS AND PATENTS PENDING.
%
randSeed=0;
rand('state',randSeed);
randn('state',randSeed);

N=600;%The number of points points in the manifold
options=[];
sigma=0.05;% Noise variance
d=2;
n_fcn='k';% Set the neighborhood function to 'k nearest neighbors'
K=10; % Set the neighborhood to 10 nearest neighbors
ratio=2; % set the axis ratio of the swiss hole
[X,TH,iD]=generateRoll3DNoiseIdealDists(N,sigma,[5,6.5,6,12,5.25,6.25,7,11],ratio);
X=X';
TH(:,2)=TH(:,2)*ratio;
options.display = 1;
%% For TCIE behaviour
options.weights=1;
%% For ISOMAP behavior
% options.weights=0;
options.ITERS=1500;
options.interpType='opt'; % Get multiscale weights using a small optimization problem
options.minDStress=1E-3;
options.min_halt_iter=5;
profile on
[Y,stats] = computeTCIE(X,K, 2,options);
s=profile('info');
