% Draws the mapping of a swiss hole using PCA, Isomap and TCIE
% By Guy Rosman, 2008
%
% for more details, see:
% "G. Rosman, A. M. Bronstein, M. M. Bronstein, R. Kimmel, Manifold
% Analysis by Topologically Constrained Isometric Embedding, World
% Enformatika Conference on Machine Learning and Pattern Recognition (MLPR), 2006
%
% ANY ACADEMIC USE OF THIS CODE MUST CITE THE ABOVE REFERENCES. 
% ANY COMMERCIAL USE PROHIBITED WITHOUT WRITTEN CONSENT OF THE AUTHORS. 
% PLEASE CONTACT THE AUTHORS FOR LICENSING TERMS. 
% PROTECTED BY INTERNATIONAL INTELLECTUAL PROPERTY LAWS AND PATENTS PENDING.
%
[x,y]=meshgrid(0:0.015:1);
rad=sqrt((x-0.5).^2+(y-0.5).^2);
idx=find(rad<0.25 | rad>0.5);
x(idx)=[];
y(idx)=[];
x=x+randn(size(x))*0.005;
y=y+randn(size(y))*0.005;

z=rand(size(x)*3)*4;
z=imfilter(z,fspecial('Gaussian',20,5));
z=z((size(x,1)+1):2*size(x,1),(size(x,2)+1):2*size(x,2));
x2=x;
y=y*20;
x=(x+1).*(x2*5+5).*sin(x2*7);
z=(z+1).*(x2*5+5).*cos(x2*7);
X=[x(:),y(:),z(:)];
plot3(X(:,1),X(:,2),X(:,3),'.');axis image
options.dims = 2;
options.display = 1;
options.weights=1;
options.ITERS=1500;
options.interpType='opt';
options.minDStress=1E-3;
options.min_halt_iter=5;
options.vector_extrapolation=1;
options.minMaxDD=1E-3;
options.MS=1;
options.verbose=0;
options2=options;
options2.weights=0; % for isomap
K=12;
profile on
[Y_TCIE,stress_TCIE] = computeTCIE(X,K, 2,options);
[Y_ISO, stress_ISO, wx,Ws,yss,stat] = computeTCIE(X,K, 2,options2);
s=profile('info');
% run PCA
[Y_PCA,SCOE_PCA,latent]=princomp(X');Y_PCA=Y_PCA*sparse(diag(latent));
close all;
plot3(X(:,1),X(:,2),X(:,3),'.');axis off;axis image;
figure
plot(Y_PCA(:,1),Y_PCA(:,2),'.');axis off;axis image;
figure
plot(Y_ISO(:,1),Y_ISO(:,2),'.');axis off;axis image;
figure
plot(Y_TCIE(:,1),Y_TCIE(:,2),'.');axis off;axis image;
